<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\YoutubeParser;
use Infomaker\Everyware\NewsML\Item;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;

class YoutubeParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(YoutubeParser::class);
    }
    public function it_should_parse_an_element()
    {
        $xml = '
            <object id="youtubeembed-b2f175bffda606e12deeb40f7aa6e962" type="x-im/youtube" url="https://www.youtube.com/watch?v=dQw4w9WgXcQ&amp;feature=youtu.be&amp;t=42" uri="https://www.youtube.com/watch?v=dQw4w9WgXcQ&amp;feature=youtu.be&amp;t=42">
                <data>
                    <start>57.2</start>
                </data>
                <links>
                    <link rel="alternate" type="text/html" url="https://www.youtube.com/watch?v=dQw4w9WgXcQ&amp;feature=youtu.be&amp;t=42" title="Click link to view content">
                        <data>
                            <context>Video</context>
                            <provider>YouTube</provider>
                        </data>
                    </link>
                    <link rel="alternate" type="image/jpg" url="https://i.ytimg.com/vi/dQw4w9WgXcQ/hqdefault.jpg">
                        <data>
                            <width>480</width>
                            <height>360</height>
                        </data>
                    </link>
                </links>
            </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                'id' => 'youtubeembed-b2f175bffda606e12deeb40f7aa6e962',
                'type' => 'x-im/youtube',
                'url' => 'https://www.youtube.com/watch?v=dQw4w9WgXcQ&feature=youtu.be&t=42',
                'uri' => 'https://www.youtube.com/embed/dQw4w9WgXcQ&feature=youtu.be&t=42',
                'thumb' => [
                    'width' => '480',
                    'height' => '360',
                    'url' => 'https://i.ytimg.com/vi/dQw4w9WgXcQ/hqdefault.jpg'
                ],
                'start' => '57.2',
                'embedId' => 'dQw4w9WgXcQ',
            ]);
    }
    public function it_should_parse_an_element_with_default_values_if_none_set()
    {
        $xml = '
            <object id="youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7" type="x-im/youtube" uri="" url="https://www.youtube.com/watch?v=htqjcfSrQsA">
                <links>
                    <link rel="alternate" type="image/jpg" url="">
                        <data>
                            <width></width>
                            <height></height>
                        </data>
                    </link>
                </links>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                'id' => 'youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7',
                'type' => 'x-im/youtube',
                'uri' => '',
                'url' => 'https://www.youtube.com/watch?v=htqjcfSrQsA',
                'thumb' => [
                    'width' => 0,
                    'height' => 0,
                    'url' => ''
                ],
                'start' => '0',
                'embedId' => '',
            ]);
    }
    public function it_should_parse_embedid_without_url_paramters()
    {   
        $xml = '
            <object id="youtubeembed-b2f175bffda606e12deeb40f7aa6e962" type="x-im/youtube" url="https://www.youtube.com/watch?v=dQw4w9WgXcQ&amp;feature=youtu.be&amp;t=42" uri="https://www.youtube.com/watch?v=dQw4w9WgXcQ">
                <data>
                    <start>0</start>
                </data>
            </object>
        ';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                'id' => 'youtubeembed-b2f175bffda606e12deeb40f7aa6e962',
                'type' => 'x-im/youtube',
                'url' => 'https://www.youtube.com/watch?v=dQw4w9WgXcQ&feature=youtu.be&t=42',
                'uri' => 'https://www.youtube.com/embed/dQw4w9WgXcQ',
                'thumb' => [],
                'start' => '0',
                'embedId' => 'dQw4w9WgXcQ',
            ]);
    }
    public function it_should_parse_an_empty_thumb_if_links_are_missing()
    {
        $xml = '
            <object id="youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7" type="x-im/youtube" uri="" url="https://www.youtube.com/watch?v=htqjcfSrQsA">
                <data>
                    <start>0</start>
                </data>
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                'id' => 'youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7',
                'type' => 'x-im/youtube',
                'uri' => '',
                'url' => 'https://www.youtube.com/watch?v=htqjcfSrQsA',
                'thumb' => [],
                'start' => '0',
                'embedId' => '',
            ]);
    }
    public function it_should_have_default_values_when_data_is_empty()
    {
        $xml = '
            <object id="youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7" type="x-im/youtube" uri="" url="https://www.youtube.com/watch?v=htqjcfSrQsA">
                <data>
                </data>
            </object>';
            $el = simplexml_load_string($xml, 'SimpleXMLIterator');
            $this->parse($el)
                ->shouldHaveArrayKey([
                    'id' => 'youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7',
                    'type' => 'x-im/youtube',
                    'uri' => '',
                    'url' => 'https://www.youtube.com/watch?v=htqjcfSrQsA',
                    'thumb' => [],
                    'start' => '0',
                    'embedId' => '',
                ]);
    }
    public function it_should_have_default_values_when_data_is_missing()
    {
        $xml = '
            <object id="youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7" type="x-im/youtube" uri="" url="https://www.youtube.com/watch?v=htqjcfSrQsA">
            </object>';
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)
            ->shouldHaveArrayKey([
                'id' => 'youtubeembed-520ee6fcf2b9d2376c41b89cf10d98d7',
                'type' => 'x-im/youtube',
                'uri' => '',
                'url' => 'https://www.youtube.com/watch?v=htqjcfSrQsA',
                'thumb' => [],
                'start' => '0',
                'embedId' => '',
            ]);
    }
    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/youtube');
    }
    public function getMatchers(): array
    {
        return [
            'haveArrayKey' => function ($subject, $values) {
                return ($subject[0]->getItemAttributes() === $values) && ($subject[0] instanceof Item);
            },
        ];
    }
}
